// Тема, в которой обсуждается этот проект, https://clck.ru/JNqKg
// link to the topic that discusses this development: https://clck.ru/JNqKg
// Крайнее редактирование 18.10.2020.
/*
  Эта версия самая компактная, специально оптимизируется под использование
  дешёвых комплектующих: плата arduino на AtMega168p / 328p(5v / 16MHz), LCD1602 без i2c,
  один модуль MAX6675, самодельная (или покупная)аналоговая клавиатура на 5 кнопок,
  самодельная (или покупная)3-4х-канальная плата реле на 12в, 2 вентилятора на 12в.
  Здесь только канал верхнего нагревателя, и термостол (нижний нагреватель)
  можно использовать любой, имеющийся в наличии, не важно, с ручным он,
  или с автоматическим управлением.
  Несмотря на наличие только канала управления верхним нагревателем, контроллер 
  имеет широкий набор функций и позволяет строить график 
  температура/время в программе Serial Port Plotter: 
  (https://github.com/CieNTi/serial_port_plotter/releases/tag/v1.2.0) на мониторе ПК.
  Настройки для Serial Port Plotter:  BAUD=9600, POINTS=600, Y STEP=20, MIN=25, MAX=250. 
  Номер com-порта в программе выберите тот, к которому подключена arduino.
  Сначала запускаем график кнопкой с треугольником, затем включаем пайку.
  При старте пайки включается вентилятор охлаждения
  симисторов и контроллера и красный мигающий светодиод.
  1) пищалка = А3 (pin.26 AtMega168p)
  2) Аналоговая клавиатура = А6 (pin.19 AtMega168p)
  3) Вход детектора ноля = 2 (pin.32 AtMega168p)
  4) выход на управление верхним излучателем = 3 (pin.1 AtMega168p)
  5) LCD1602: ( rs = 4, en = 5, d4 = 6, d5 = 7, d6 = 8, d7 = 9 )(pin's.2,9,10,11,12,13 AtMega168p)
  6) пин flash LED и охладителя симисторов и контроллера = A2 (pin.25 AtMega168p)
  7) преобразователь термопары MAX6675:
  SO = 12 (DO=(SO), pin.16 AtMega168p)
  CStop = 11 (CS, pin.15 AtMega168p)
  CLK = 10 (SCK=(CLK), pin.15 AtMega168p)
  8) пин охладителя платы = А5 (pin.28 AtMega168p)
  9) пин управления включением и отключением термостола = А0 (pin.23 AtMega168p)
  Выражаю огромную благодарность участнику форума Амперки, DmitrySh.
  DmitrySh дал жизнь этому проекту, исправил ошибки, заменил
  громоздкие подключаемые библиотеки функциями, упорядочил код.
*/
/*
  https://clck.ru/JyBRt
  1. автономный термостол включается при старте пайки и выключается после неё пином А0
  (сколько угодно раз, активный уровень низкий).
  2. Верх включится по нажатию ВПРАВО дольше 3 сек, или
  по достижении на термопаре 120 градусов.
  3. после окончания профиля автоматически НЕ ВКЛЮЧИТСЯ охладитель платы пином А5
  (выход на реле вентиляторов охлаждения).
  4. выключить охладитель платы можно из режима IDLE клавишей CANCEL.
  5. включить охладитель платы из режима IDLE можно коротким нажатием
  клавиши ВПРАВО(это на случай если вы прервали профиль, но плата горячая
  и вам необходимо её по-быстрому охладить или просто жарко
  и охладитель используется как вентилятор:)).
  6. Теперь профиль хранится в виде структуры, что уменьшает
  занимаемое скетчем место и сохраняет EEPROM от лишних перезаписей.
  7. параметры сохраняются в EEPROM после каждого нажатия OK, т.е.,
  если вам нужно поменять один параметр, не придётся пролистывать весь список до конца -
  просто меняем параметр, жмём ОК и CANCEL.
  8. Немного оптимизированы код и графика для "расчистки места работы".
  9. Логика ПИД ВИ заменена на предложенную Dmitrysh в посте #3009:
  "работать не по конечному значению ошибки, а по измерению температуры.".
  Эта логика особенно актуальна для "быстрых" излучателей (галогенки, открытая спираль, кварцевая панель).
  Также присутствует и классический вариант расчета, он закоментиован.
  При настройке параметров ПИД обращайте внимание на коэффициенты деления/умножения 
  составляющих D и I внутри функции Pid1 - она находится в конце скетча!!! Если будет много, или мало какой-то составляющей, 
  просто измените коэффициент деления/умножения.
  10. Класс Serial довольно прожорливый, для экономии заменён на small UART.
  11. Состояние для attachInterrupt нужно задать под свою версию схемы (ZCC) контроля
  перехода напряжения сети через ноль (FALLING или RISING).
  12. Профилей можно задать 4, максимальное число шагов каждого профиля ограничено 4.
  13. Мощность излучателя, как минимальную, так и максимальную, можно задать для каждого шага.
  14. Добавлена проверка на корректность ввода температуры по шагам,- последующая
  температура не должна быть меньше предыдущей.
  15. Во второй строке при задании мощности ВИ по шагам выводится номер шага.
  16. Т.к. используется аналоговая (резистивный делитель) клавиатура, значения для кнопок в блоке
  [  
  { PIN_RIGHT, 408},
  { PIN_UP, 811},
  { PIN_DOWN, 611},
  { PIN_LEFT, 328},
  { PIN_SELECT, 216}
  ] 
  нужно выставить именно для своего экземпляра клавиатуры. Пример вычисления значений:
  http://arduinolab.pw/index.php/2015/12/05/rezistivnaya-klaviatura-princip-raboty/
*/
/*
  Замер и вывод температуры на дисплей и в com-порт выделены в отдельную функцию 
  и вызываются 2 раза в скетче - уменьшился размер кода.
  Прокрутка значений параметров профиля стала удобнее.
  Исправлено отображение параметров на дисплее при их редактировании.
*/
  
//---ProMini_LCD1602_ANALOG KEYBOARD_15-07-2020---//
#include <EEPROM.h>
#include <LiquidCrystal.h>
const int rs = 4, en = 5, d4 = 6, d5 = 7, d6 = 8, d7 = 9;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

#define UART_DOUBLESPEED
#define UART_TXREADY UDRE0
#define UART_RXREADY RXC0
#define UART_DOUBLE  U2X0
#define UDR  UDR0
#define UCRA UCSR0A
#define UCRB UCSR0B
#define UCRC UCSR0C
#define UCRC_VALUE ((1<<UCSZ01) | (1<<UCSZ00))
#define RXCIE RXCIE0
#define TXCIE TXCIE0
#define RXEN RXEN0
#define TXEN TXEN0
#define UBRRL UBRR0L
#define UBRRH UBRR0H
#define UART_CALC_BAUDRATE(baudRate) ((uint32_t)((F_CPU) + ((uint32_t)baudRate * 4UL)) / ((uint32_t)(baudRate) * 8UL) - 1)

//---секция ПИД---//
float integra, e, p, d;             //интегральная составляющая, ошибка регулирования, П-составляющая, Д-составляющая
#define i_min 0.0                   //минимум И составляющей
#define i_max 100.0                 //максимум И составляющей
double  Input1;                     //текущая температура(измеренная max6675) ВИ
byte Output1;                       //выход ПИД регулятора ВИ
//---секция ПИД---//

byte buf[5]; //это для графиков на ПК

void make_buf(byte temp)
{ byte i, small, dig, ch;
  for (i = 0; i <= 2; i++)
  {
    small = temp / 10;
    dig = temp - small * 10 ;       //получаем ASSCI код символа
    ch = dig + 48 ;
    buf[3 - i] = ch;                //получается , что мы записываем целую часть начиная с младшего символа
    temp = temp / 10;
  }
}

//---секция алгоритма Брезенхема---//
int er1 = 1;                        //переменная для расчета переполнения в алгоритме Брезенхема для ВИ
int reg1;                           //промежуточная переменная для алгоритма Брезенхема для ВИ
boolean out1;                       //выход импульса управления ВИ алгоритма Брезенхема
#define SENSOR_SAMPLING_TIME 250    //частота обновления текущей температуры (1000=1 раз в секунду)
//---секция алгоритма Брезенхема---//

//---секция кнопок и пинов подключения---//
//---настройка аналоговых кнопок управления----//
//аналоговая клавиатура на А6
//#define A_PINS_BASE 100           //номер с которого начинается нумерация наших "псевдо-кнопок".
#define PIN_RIGHT 100
#define PIN_UP 101
#define PIN_DOWN 102
#define PIN_LEFT 103
#define PIN_SELECT 104

struct A_PIN_DESC {                 //определяем  структуру которой будем описывать какое значение мы ожидаем для каждого псевдо-пина
  byte pinNo;                       //номер пина
  int expectedValue;                //ожидаемое значение
};
A_PIN_DESC expected_values[] = {                                //ожидаемые значения для псевдо-кнопок
  { PIN_RIGHT, 408},
  { PIN_UP, 811},
  { PIN_DOWN, 611},
  { PIN_LEFT, 328},
  { PIN_SELECT, 216}
};
#define A_PINS_COUNT sizeof(expected_values)/sizeof(A_PIN_DESC) //вычисляем сколько у нас всего "псевдо-кнопок" заданно.
#define A_POSSIBLE_ABERRATION 20                                //допустимое отклонение analogRead от ожидаемого значения, при котором псевдо кнопка считается нажатой
bool digitalReadA(byte pinNo) {
  for (byte i = 0; i < A_PINS_COUNT; i++) {                     //ищем описание нашего всевдо-пина
    A_PIN_DESC pinDesc = expected_values[i];                    //берем очередное описание
    if (pinDesc.pinNo == pinNo) {                               //нашли описание пина?
      int value = analogRead(A6);                               //производим чтение аналогого входа А6
      return (abs(value - pinDesc.expectedValue) < A_POSSIBLE_ABERRATION); //возвращаем HIGH если отклонение от ожидаемого не больше чем на A_POSSIBLE_ABERRATION
    }
  }
  return LOW;                                                   //если не нашли описания - считаем что пин у нас LOW
}
//Назначаем пины кнопок управления
int upSwitchPin = PIN_UP;
int downSwitchPin = PIN_DOWN;
int cancelSwitchPin = PIN_LEFT;
int okSwitchPin = PIN_SELECT;
int rightSwitchPin = PIN_RIGHT;
//состояние кнопок по умолчанию
boolean upSwitchState = 0;
boolean downSwitchState = 0;
boolean cancelSwitchState = 0;
boolean okSwitchState = 0;
boolean rightSwitchState = 0;
//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;

#define RelayPin1 3                //назначаем пин "ВЕРХНЕГО" нагревателя
#define buzzerPin A3               //назначаем пин пищалки
#define Int_Fan  A2                //назначаем пин охладителя симисторов и контроллера
#define Ext_Fan  A5                //назначаем пин охладителя платы
#define BH_OFF  A0                 //назначаем пин отключателя НИ после завершения профиля

//---назначаем пины для модуля MAX6675 - преобразователя сигнала термопары---//
#define thermoCLK  10              //SCK=(CLK)
#define thermoCStop  11            //CS 
#define thermoSO  12               //DO=(SO)
//---секция кнопок и пинов подключения---//

//---секция профиля---//
struct pr {                        // основные поля профиля
  byte profileSteps;               //количество шагов профиля
  byte min_pwr_TOPStep[4];         //минимальная мощность верхнего нагревателя
  byte max_pwr_TOPStep[4];         //максимальная мощность верхнего нагревателя
  byte rampRateStep[4];            //скорость ростsetpointRampа температуры
  byte temperatureStep[4];         //заданные температуры для шагов профиля ВИ
  byte dwellTimerStep[4];          //установленное время перехода на следубщий шаг
  byte kp1;                        //пропорциональный коэффициент ПИД для ВИ
  byte ki1;                        //интегральный коэффициент ПИД для ВИ
  byte kd1;                        //дифференциальный коэффициент ПИД для ВИ
};
byte SizeProfile = sizeof(pr);     // длина поля данных
pr profile;
byte currentProfile = 1;           //текущий профиль
byte editStep = 0;                 //номер шага профиля который редактируется в данный момент
byte currentStep = 1;              //текущий шаг профиля
byte Setpoint1;                    //установленная температура(уставка) ВИ
//---секция профиля---//

//---Секция флагов---//
boolean TopStart = false;          //флаг включения ВИ
boolean flag = 0;                  //флаг для фиксации стартовой температуры и начала работы нагревателей
byte x = 1;                        //переменная для перехода на нужный шаг при горячей плате
boolean alarmOn = false;           //признак предупреждения
boolean updateScreen = true;       //признак обновления экрана
//---Секция флагов---//

//---Секция переменных общего назначения---//
byte startTemp;                    //стартовая температура для профиля, когда НИ вышел на уставку, используется для определения шага профиля с которого начинаем
byte setpointRamp;                 //заданная температура по определённому шагу профиля ВИ, связана с заданной скоростью роста температуры
byte counter;                      //счётчик для рампы и счетчик для времени нахождения в шаге
unsigned long previousMillis;      //это для счетчиков
unsigned long nextRead2;           //переменная для обновления текущей температуры и расчёта ПИД
int tc1;                           //переменная для хранения текущей температуры ВИ
int i = 0;
//---Секция переменных общего назначения---//

//---структура статусов событий---//
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE : byte
{
  REFLOW_STATE_IDLE,               //экран в режиме простоя, выбор номера профиля
  REFLOW_STATE_MENU_STEPS,         //меню выбора числа шагов редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR_MIN,   //меню выбора мин. мощности ВИ в шагах редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR_MAX,   //меню выбора макс. мощности ВИ в шагах редактируемого профиля
  REFLOW_STATE_MENU_STEP_RAMP,     //меню выбора скорости роста t° ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_STEP_TARGET,   //меню выбора конечной t° на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR,       //меню выбора мощности ВИ редактируемого профиля
  REFLOW_STATE_MENU_STEP_DWELL,    //меню выбора времени удержания конечной t° на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_TOP_P,         //меню выбора коэффициента P для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_I,         //меню выбора коэффициента I для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_D,         //меню выбора коэффициента D для ВИ редактируемого профиля
  REFLOW_STATE_STEP_RAMP,          //"процесс пайки", рост температуры с заданной для текущего шага ВИ скоростью
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE
}
reflowState_t;
typedef enum REFLOW_STATUS : byte  //структура статуса пайки, запущен или нет
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;
reflowStatus_t reflowStatus;       //переменная статусов событий
reflowState_t reflowState;         //переменная статуса пайки
//---структура статусов событий---//

void loadProfile()//this function loads whichever profile currentProfile variable is set to
{
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.get((currentProfile - 1)*SizeProfile, profile);
  }
}
void SaveProfile () {              //сохранение текущего профиля
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.put((currentProfile - 1)*SizeProfile, profile);
  }
}

void UART_SendByte(uint8_t data)
{
  while (!(UCRA & (1 << UART_TXREADY)));
  UDR = data;
}

void UART_SendArray(uint8_t *buffer, uint8_t bufferSize)
{
  for (uint8_t i = 0; i < bufferSize; i++)
    UART_SendByte(buffer[i]);
}

void UART_Init(uint32_t UART_BAUD_RATE)
{
  UBRRH = (UART_CALC_BAUDRATE(UART_BAUD_RATE) >> 8) & 0xFF;
  UBRRL = (UART_CALC_BAUDRATE(UART_BAUD_RATE) & 0xFF);
  UCSR0A = ( 1 << UART_DOUBLE );
  UCRB = ((1 << TXEN) | (1 << RXEN));
  UCRC = UCRC_VALUE;
}

void setup()
{
  UART_Init(9600);
  buf[0] = 36;
  buf[4] = 59;

  loadProfile();                                //вызов функции loadProfile для загрузки данных профиля из eeprom

  //attachInterrupt(0, Dimming, RISING);        //настроить порт прерывания - 2й цифровой пин
  attachInterrupt(0, Dimming, FALLING);         //настроить порт прерывания - 2й цифровой пин
  /* -------------------------------------------------------------------------------
    LOW вызывает прерывание, когда на порту LOW
    CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
    RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
    FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW
    ------------------------------------------------------------------------------- */

  //---задаём состояние пинов, к которым подключены MAX6675---//
  pinMode(thermoCStop, OUTPUT);
  pinMode(thermoCLK, OUTPUT);
  pinMode(thermoSO, INPUT);

  //---устанавливаем пин пищалки на ВЫХОД---//
  pinMode(buzzerPin, OUTPUT);

  //---устанавливаем пины охладителей на ВЫХОД---//
  pinMode (Int_Fan, OUTPUT);
  pinMode (Ext_Fan, OUTPUT);
  pinMode (BH_OFF, OUTPUT);
  //---задаём состояние пинов управления выходами---//
  pinMode(RelayPin1, OUTPUT);
  
  lcd.begin(16, 2);
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("ARDUINO REWORK");
  lcd.setCursor(0, 1);
  lcd.print("v1.4_Top Channel");
  delay (3000);

  /*  //Мелодия приветствия
    tone(buzzerPin, 523);
    delay(200);
    tone(buzzerPin, 659);
    delay(200);
    tone(buzzerPin, 784);
    delay(200);
    tone(buzzerPin, 1046);
    delay(200);
    noTone(buzzerPin); */

  //lcd.begin(16, 2);
}

//---основной цикл программы---//
void loop()
{
  //---Считываем состояние кнопок управления---//
  upSwitchState = digitalReadA(upSwitchPin);
  downSwitchState = digitalReadA(downSwitchPin);
  cancelSwitchState = digitalReadA(cancelSwitchPin);
  okSwitchState = digitalReadA(okSwitchPin);
  rightSwitchState = digitalReadA(rightSwitchPin);

  unsigned long currentMillis = millis();
  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      i++;
    } else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
      TopStart = false;
      digitalWrite(BH_OFF, HIGH);                             //выключить НИ
      digitalWrite(Int_Fan, LOW);                             //выключить охладитель симисторов
      currentStep = 1;
      counter = 0;
      setpointRamp = 0;
      x = 1;                                                  //устанавливаем переменную в исходное состояние
      flag = 0;                                               //после отановки профиля сбрасываем флаг
      Output1 = 0;
      if (millis() > nextRead2)
      {
        lcdTemp();
        //---считываем информацию с преобразователя термопары---//
        nextRead2 = millis() + SENSOR_SAMPLING_TIME;          //устраняем колебания датчиков температуры
        //Input1  = Input1 * 0.8 + 0.2 * (max6675_read_temp(thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
        //tc1 = Input1;
        //make_buf(byte(tc1));
        //UART_SendArray(buf, 5);
        //lcd.setCursor(13, 1);
        //lcd.print("    ");
        //lcd.setCursor(13, 1);
        //if (isnan(Input1)) {
        //lcd.print("Er");
        //} else {
        // lcd.print(tc1);
        //}
      }

      //Обновляем экран один раз
      if (updateScreen) {
        //Настройка экрана в режиме ожидания
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("OFF");
        lcd.setCursor(4, 0);
        lcd.print("PTN:");
        lcd.print("  STEP:1");
        lcd.setCursor(0, 1);
        lcd.print("TH SP:");
        lcd.setCursor(10, 1);
        lcd.print("PV:");
        updateScreen = false;
      }
      lcd.setCursor(8, 0);
      lcd.print(currentProfile);
      lcd.print(" ");
      lcd.setCursor(6, 1);
      lcd.print(profile.temperatureStep[0]);
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 500)   //когда нажата ВВЕРХ - перейти к следующему профилю
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 5)                                    //когда текущий профиль = 4 и нажата ВВЕРХ - вернуться к профилю 1
        {
          currentProfile = 1;
        }
        loadProfile(); //вызов функции loadProfile для загрузки данных профиля из eeprom
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 500) //когда нажата ВНИЗ - вернуться на предыдущий профиль
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 4;
        }
        loadProfile(); //вызов функции loadProfile для загрузки данных профиля из eeprom
      }

      //---фиксируем момент нажатия кнопки "ОК" + защита от дребезга---//
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }

      //---держим "ОК" 3 секунды и заходим в меню настроек---//
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button) > 3000)
      {
        button_long_state = true;
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEPS;
      }

      //---включаем пайку, кнопка сработает после отпускания "ОК" если нажатие меньше 3х секунд---//
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        tone(buzzerPin, 1045, 500);                           //звуковой сигнал при старте профиля
        //---обновить экран---//
        updateScreen = true;
        nextRead2 = millis();
        digitalWrite(Ext_Fan, LOW);                           //выключить охлаждение платы
        digitalWrite(Int_Fan, HIGH);                          //включить охладитель симисторов и контроллера
        digitalWrite(BH_OFF, LOW);                            //включить НИ
        integra = 0;
        p = 0;
        d = 0;
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
        updateScreen = true;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Ext_Fan, LOW);
      }
      if (rightSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Ext_Fan, HIGH);
      }
      break;

    //---устанавливаем количество шагов профиля---//
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen) {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("PTN ");
        lcd.print(currentProfile);
        lcd.print(" Edit");
        lcd.setCursor(0, 1);
        lcd.print("PTN Steps:");
        updateScreen = false;
      }
      lcd.setCursor(10, 1);
      lcd.print(profile.profileSteps);

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.profileSteps = profile.profileSteps + 1;
        if (profile.profileSteps >= 5) {
          profile.profileSteps = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.profileSteps = profile.profileSteps - 1;
        if (profile.profileSteps <= 0)
        {
          profile.profileSteps = 4;
        }
      }
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню кнопка "ОК" не считалась нажатой

      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_PWR_MIN;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_TOP_PWR_MIN:
      if (updateScreen)
      {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("min PWR:");
        lcd.setCursor(0, 1);
        lcd.print("St: ");             
        lcd.print(editStep + 1);
        updateScreen = false;
      }
      lcd.setCursor(8, 0);
      lcd.print(profile.min_pwr_TOPStep[editStep]);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.min_pwr_TOPStep[editStep] = profile.min_pwr_TOPStep[editStep] + 1;
        if (profile.min_pwr_TOPStep[editStep] >= 101)
        {
          profile.min_pwr_TOPStep[editStep] = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        if (profile.min_pwr_TOPStep[editStep] <=0)
        {
          profile.min_pwr_TOPStep[editStep] = 99;
        }
        else profile.min_pwr_TOPStep[editStep] = profile.min_pwr_TOPStep[editStep] - 1;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps)
        {
          editStep = 0;
          SaveProfile ();
          reflowState = REFLOW_STATE_MENU_TOP_PWR_MAX;
        } else editStep++;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_TOP_PWR_MAX:
      if (updateScreen)
      {
        lcd.setCursor(0, 0);
        lcd.print("max PWR:");
        lcd.setCursor(0, 1);
        lcd.print("St: ");
        lcd.print(editStep + 1);
        updateScreen = false;
      }
      lcd.setCursor(8, 0);
      lcd.print(profile.max_pwr_TOPStep[editStep]);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.max_pwr_TOPStep[editStep] = profile.max_pwr_TOPStep[editStep] + 1;
        if (profile.max_pwr_TOPStep[editStep] >= 101)
        {
          profile.max_pwr_TOPStep[editStep] = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        if (profile.max_pwr_TOPStep[editStep] <=0)
        {
          profile.max_pwr_TOPStep[editStep] = 99;
        }
        else profile.max_pwr_TOPStep[editStep] = profile.max_pwr_TOPStep[editStep] - 1;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps)
        {
          editStep = 0;
          SaveProfile ();
          reflowState = REFLOW_STATE_MENU_STEP_RAMP;
          lcd.setCursor(7, 1);
          lcd.print("      ");
        } else editStep++;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---устанавливаем скорость нагрева "Верхним Нагревателем"---//
    case REFLOW_STATE_MENU_STEP_RAMP:
      if (updateScreen) 
      {
        lcd.setCursor(0, 0);
        lcd.print("Parameter Editor");
        lcd.setCursor(0, 1);
        lcd.print("St ");
        lcd.print(editStep + 1);
        lcd.print(" Ramp:     ");
        updateScreen = false;
      }
      lcd.setCursor(10, 1);
      lcd.print(profile.rampRateStep[editStep]);
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.rampRateStep[editStep] = profile.rampRateStep[editStep] + 1;
        if (profile.rampRateStep[editStep] >= 30)                          //(только целые числа) максимальная скорость роста температуры умноженная на 10
        {
          profile.rampRateStep[editStep] = 30;                             //(только целые числа) максимальная скорость роста температуры умноженная на 10
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.rampRateStep[editStep] = profile.rampRateStep[editStep] - 1;
        if (profile.rampRateStep[editStep] <= 1)                           //(только целые числа) минимальная скорость роста температуры умноженная на 10
        {
          profile.rampRateStep[editStep] = 1;                              //(только целые числа) минимальная скорость роста температуры умноженная на 10
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
          SaveProfile();
        } else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---устанавливаем температуру "Верхнего Нагревателя"---//
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("St ");
        lcd.print(editStep + 1);
        lcd.print(" Target:  ");
        updateScreen = false;
      }
      lcd.setCursor(12, 1);
      lcd.print(profile.temperatureStep[editStep]);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.temperatureStep[editStep] = profile.temperatureStep[editStep] + 1;
        if (profile.temperatureStep[editStep] >= 256)
        {
          profile.temperatureStep[editStep] = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.temperatureStep[editStep] = profile.temperatureStep[editStep] - 1;
        if (profile.temperatureStep[editStep] <= 0)
        {
          profile.temperatureStep[editStep] = 254;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        } else {
          editStep++;
          if (profile.temperatureStep[editStep] < profile.temperatureStep[editStep - 1])
            profile.temperatureStep[editStep] = profile.temperatureStep[editStep - 1];
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---устанавливаем время перехода на следующий шаг---//
    case REFLOW_STATE_MENU_STEP_DWELL:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("St ");
        lcd.print(editStep + 1);
        lcd.print(" Dwell:");
      }
      lcd.setCursor(11, 1);
      lcd.print(profile.dwellTimerStep[editStep]);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] + 5;
        if (profile.dwellTimerStep[editStep] >= 180)
        {
          profile.dwellTimerStep[editStep] = 180;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();

        if (profile.dwellTimerStep[editStep] < 5)
        {
          profile.dwellTimerStep[editStep] = 0;
        }
        else profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] - 5;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps)
        {
          editStep = 0;
          SaveProfile();
          reflowState = REFLOW_STATE_MENU_TOP_P;
        } else editStep++;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---настройка "ПИД" верхнего нагревателя---//
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("TH:            ");
        lcd.setCursor(4, 1);
        lcd.print("P=");
        updateScreen = false;
      }
      lcd.setCursor(6, 1);
      lcd.print(profile.kp1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kp1 = profile.kp1 + 1;
        if (profile.kp1 >= 256)
        {
          profile.kp1 = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();

        profile.kp1 = profile.kp1 - 1;
        if (profile.kp1 <= 0)
        {
          profile.kp1 = 254;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
      lcd.setCursor(4, 1);
      lcd.print("I=");
      lcd.print(profile.ki1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki1 = profile.ki1 + 1;
        if (profile.ki1 >= 256)
        {
          profile.ki1 = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki1 = profile.ki1 - 1;
        if (profile.ki1 <= 0)
        {
          profile.ki1 = 254;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_D;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_D:
      lcd.setCursor(4, 1);
      lcd.print("D=");
      lcd.print(profile.kd1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd1 = profile.kd1 + 1;
        if (profile.kd1 >= 256)
        {
          profile.kd1 = 0;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd1 = profile.kd1 - 1;
        if (profile.kd1 <= 0)
        {
          profile.kd1 = 254;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        SaveProfile();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---"процесс пайки", рост температуры с заданной скоростью---//
    case REFLOW_STATE_STEP_RAMP:
      digitalWrite(Int_Fan, HIGH);                                     //включить охладитель симисторов и контроллера
      if (updateScreen) {
        lcd.setCursor(0, 0);
        lcd.print("RUN");
        updateScreen = false;
      }
      //--------включение ВИ кнопкой ВПРАВО-------------//
      //---фиксируем факт нажатия клавиши ВПРАВО---//
      if (rightSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }
      //---держим "ВПРАВО" 3 секунды и включаем верхний нагреватель---//
      if (rightSwitchState == HIGH && !button_long_state && ( millis() - ms_button) > 3000)
      {
        button_long_state = true;
        button_state = false;
        TopStart = true;
      }
      //---если нажатие кратковременное сбрасываем флаг фиксации факта нажатия---//
      if (rightSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
      }
      //--------включение ВИ кнопкой ВПРАВО-------------//

      //if (tc1 >= 35 && !TopStart) TopStart = true;
      if (tc1 >= 120 && !TopStart) TopStart = true;                     //если температура на датчике верха достигла 120*С включаем верхний нагреватель
      if (TopStart == true)                                             //если включаем верхний нагреватель:
        //-------------------------------------------------------------------------------------------
      { //открываем скобку (чтоб не запутаться)

        //---фиксируем стартовую температуру---//
        if (flag == 0)
        {
          startTemp = tc1;
          flag = 1;
        }

        //---устанавливаем нужный шаг, до которой нагрета плата---//
        if (startTemp > profile.temperatureStep[currentStep - 1]) {
          for (x = 1; startTemp > profile.temperatureStep[currentStep - 1]; currentStep++) {
            x++;
          }
        }
        if (currentStep > x && flag == 1) {
          flag = 0;
          startTemp = profile.temperatureStep[currentStep - 2];
          flag = 1;
        }
        lcd.setCursor(15, 0);
        lcd.print(currentStep);

        //---счётчик скорости роста температуры---//
        if ((currentMillis - previousMillis) > 1000 / (profile.rampRateStep[currentStep - 1] * 0.1)) //скорость роста температуры от 0.1с. до 3с.
        {
          previousMillis = currentMillis;
          counter = counter + 1;
          setpointRamp = counter + startTemp;
          lcd.setCursor(6, 1);
          lcd.print(" ");
          lcd.setCursor(6, 1);
          lcd.print(setpointRamp);
          lcd.print(" ");
          Setpoint1 = setpointRamp;
        }
      } //закрывам скобку
      //----------------------------------------------------------------------------------------

      if (setpointRamp >= profile.temperatureStep[currentStep - 1]) //если достигли нужной температуры
      {
        lcd.setCursor(6, 1);
        lcd.print(profile.temperatureStep[currentStep - 1]);
        reflowState = REFLOW_STATE_STEP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;
    case REFLOW_STATE_STEP:
      Setpoint1 = profile.temperatureStep[currentStep - 1];
      if (Input1 >= profile.temperatureStep[currentStep - 1])
      {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---считаем время перехода на следующий шаг---//
    case REFLOW_STATE_STEP_DWELL:
      if (currentMillis - previousMillis > 1000)
      {
        previousMillis = currentMillis;
        counter = counter + 1;
      }
      if (counter >= profile.dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
      {
        counter = 0;
        setpointRamp = 0;
        if (profile.profileSteps == currentStep) //если достигли последнего шага
        {
          currentStep = 1;
          x = 1;                                 //устанавливаем переменную в исходное состояние
          flag = 0;                              //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
        } else {                                 //если шаг не последний
          currentStep++;                         //переходим на следующий шаг
          reflowState = REFLOW_STATE_STEP_RAMP;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //---завершение пайки---//
    case REFLOW_STATE_COMPLETE:
      //digitalWrite(Ext_Fan, HIGH);             //включить охладитель платы
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;
      updateScreen = true;
      break;
  }

  //---включение нагревателей---//
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    if (millis() > nextRead2)
    {
      lcdTemp();
      //---считываем данные с термопар---//
      //Input1  = Input1 * 0.8 + 0.2 * (max6675_read_temp(thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
      //tc1 = Input1;
      if (TopStart) Output1 = Pid1(Input1, Setpoint1, profile.kp1, profile.ki1, profile.kd1); else Output1 = 0;
      nextRead2 = millis() + SENSOR_SAMPLING_TIME;

      //make_buf(byte(tc1));
      //UART_SendArray(buf, 5);
      //lcd.setCursor(13, 1);
      //lcd.print("   ");
      //lcd.setCursor(13, 1);
      //if (isnan(Input1)) {
      //lcd.print("Er");
      //} else {
      //lcd.print(tc1);
      //}
    }
  } else {
    digitalWrite(RelayPin1, LOW);
  }
}

void Dimming()
{
  static byte Secs;
  if (Secs >= 100)                  //каждые 100 прерываний отправляем данные через small UART
  {
    UART_SendByte(byte(Input1));
    Secs = 1;                       //Начинаем считать с 1, т.к. в первый шаг мы только присваиваем значение, но не прибавляем
  } else Secs++;
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    Outpwr_TOP();
  }
}

void Outpwr_TOP()
{
  reg1 = Output1 + er1;             //er- ошибка округления
  if (reg1 < 50)
  {
    out1 = LOW;
    er1 = reg1 ;                    //reg- переменная для расчетов
  } else {
    out1 = HIGH;
    er1 = reg1 - 100;
  }
  digitalWrite(RelayPin1, out1);    //пин через который осуществляется дискретное управление
}

/*  byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e = (ust - temp);                 //ошибка регулирования
  p =  (kP * e);                    //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e) / 100; //И составляющая
  d = kd * (e - ed) * 10;           //Д составляющая
  ed = e;
  out = (p + integra + d < profile.min_pwr_TOPStep[currentStep - 1]) ? profile.min_pwr_TOPStep[currentStep - 1] : (p + integra + d > profile.max_pwr_TOPStep) ? profile.max_pwr_TOPStep[currentStep - 1] : p + integra + d;
  return out;
}  */

byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e = (ust - temp);                            //ошибка регулирования
  p =  (kP * e);                               //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e) / 1000; //И составляющая
  d = kd * (temp - ed);                        //Д составляющая
  ed = temp;
  out = (p + integra - d < profile.min_pwr_TOPStep[currentStep - 1]) ? profile.min_pwr_TOPStep[currentStep - 1] : (p + integra - d > profile.max_pwr_TOPStep[currentStep - 1]) ? profile.max_pwr_TOPStep[currentStep - 1] : p + integra - d;
  return out;
}

//---MAX6675 functions---//
double max6675_read_temp (int ck, int cs, int so)
{ char i;
  int tmp = 0;
  digitalWrite(cs, LOW);//cs = 0;   //Stop a conversion in progress
  asm volatile (" nop"  "\n\t");
  for (i = 15; i >= 0; i--)
  { digitalWrite(ck, HIGH);
    asm volatile (" nop"  "\n\t");
    if ( digitalRead(so))
      tmp |= (1 << i);
    digitalWrite(ck, LOW);
    asm volatile (" nop"  "\n\t");
  }
  digitalWrite(cs, HIGH);
  if (tmp & 0x4) {
    return NAN;
  } else
    return ((tmp >> 3)) * 0.25;
}
//---MAX6675 functions---//

void lcdTemp()
{
//---считываем данные с термопар---//
      Input1  = Input1 * 0.8 + 0.2 * (max6675_read_temp(thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
      tc1 = Input1;
      //if (TopStart) Output1 = Pid1(Input1, Setpoint1, profile.kp1, profile.ki1, profile.kd1); else Output1 = 0;
      //nextRead2 = millis() + SENSOR_SAMPLING_TIME;

      make_buf(byte(tc1));
      UART_SendArray(buf, 5);
      lcd.setCursor(13, 1);
      lcd.print("   ");
      lcd.setCursor(13, 1);
      if (isnan(Input1)) {
        lcd.print("Er");
      } else {
        lcd.print(tc1);
      }
    }
